(** Identifier utilities  

Time-stamp: <modified the 05/09/2007 (at 14:54) by Erwan Jahier> 
    
    @author Louis Mandel
    @author Erwan Jahier
*)


(** {2 Identifier} *)

type t = 
  | Ident of string 
  | Dot of string * t
  | Col of string * t
(** For example the AADL name [FOO::BAR.f] is represented as 
    [Col("FOO",Dot("BAR",Ident "f"))].  
*)

val of_string : string -> t
val to_string : t -> string

(** [make_add_prefix id1 id2] adds [id1] as prefix of [id2]. *)
val make_add_prefix:  t -> t -> t

(** [add_suffix suff id] returns [id] suffixed by [suff].  *)
val add_suffix:string -> t -> t

(** [make_new_ident comp suffix] returns an identifier based on
    [comp] name and the suffix [suffix].
*)
val make_new_ident :  t -> string -> t

(** {2 Integer} *)

type integer = 
  | Id of t
  | I of int

val integer_to_string:  integer -> string


(** {2 Identifier manipulations} *)

(** aaxl idents are a bit clumsy. Here are some functions to clean them up. *)

val connection_string_to_ident : string -> string -> t
val aaxl_string_to_ident : string -> t
val aaxl_string_to_short_ident : string -> t
val aaxl_string_to_long_ident : string -> t


(** The argument ougth to be a component  *)
val feature_name_of_component_name:  t -> t


(** The argument ougth to be a component implementation *)
val type_name_of_implementation_name:  t -> t

(** [is_prefix id1 id2] checks if [id1] is a prefix of [id2]. *)
val is_prefix:  t -> t -> bool



