(* Time-stamp: <modified the 03/09/2007 (at 14:21) by Erwan Jahier> *)

open Ident
open Aadl_ast

(* exported *)
let (get_period:  Aadl_ast.component -> int) =
  fun component ->
    let period_id = Ident "Period" in
    let _, _, v, _ = 
      List.find (fun (id, _, _, _) -> id = period_id) component.comp_prop
    in
      begin match v with
	  (* XXX do not take the unit into account (cf p 221). *)
	| [ Prop_expr_immediate 
	      (Imm_const (Imm_int (Int (period, _ (* Some (Ident "Ms") *) )))) ] -> 
	    period
	| [] -> assert false
	| _ -> assert false
      end

(* exported *)
let (get_compute_execution_time:  Aadl_ast.component -> 
      Ident.integer * Ident.integer) =
  fun component ->
    try
      let exec_time_id = Ident "Compute_Execution_Time" in
      let _, _, v, _ = 
	List.find (fun (id, _, _, _) -> id = exec_time_id) component.comp_prop
      in
      let min_exec_time, max_exec_time = 
	begin match v with
	  | (Prop_expr_integer_range
	       (Int (min, _ (* Some (Ident "Ms") *) ),
		Int (max, _ (* Some (Ident "Ms") *) ),
		None))::_ -> 
	      (min, max)
	  | (Prop_expr_immediate(Imm_const(Imm_int(
					     Int (i, _ (* Some (Ident "Ms") *) )
					       
					   ))))::_ -> 
	      (i,i)
	  | [] -> assert false
	  | _ -> assert false
	end
      in
	Ident.I min_exec_time, Ident.I max_exec_time
    with
      | Not_found -> 
	  (Ident.Id 
	     (make_new_ident component.comp_id "_min_exec_time"),
	   Ident.Id 
	     (make_new_ident component.comp_id "_max_exec_time"))

(* exported *)
let (get_processor_binding:  Aadl_ast.component ->
      (Ident.t * (Ident.t * Ident.t)) list) =
  fun component ->
    let prop_id = Ident "Actual_Processor_Binding" in
    let l =
      List.find_all (fun (id, _, _, _) -> id = prop_id) component.comp_prop
    in
    let (loc_id__comp_name:(Ident.t * Ident.t) list) =
      List.fold_left
	(fun acc subc -> 
	   let (sc_loc_id, _, _, _) = subc in
	     (sc_loc_id, component.comp_id)::acc
	)
	[]
	component.comp_subc
    in
    let poc_pes_l =
      List.fold_left
	(fun acc (prop_name, prop_arrow, prop_val, prop_applies_to)  -> 
	   assert (prop_val <> []);
	   match (List.hd prop_val), prop_applies_to with
	       Prop_expr_reference_term processor_id, Some process_id ->
		 (processor_id, process_id)::acc
	     | Prop_expr_reference_term id1, None -> assert false
	     | _, _  -> assert false
	)
	[]
	l
    in
      List.map
	(fun (poc,pes) -> (poc,(pes, List.assoc pes loc_id__comp_name)))
	poc_pes_l


type dispatch_protocol =
  | Periodic
  | Aperiodic
  | Sporadic
  | Backgroud

let string_to_dispatch_protocol =
  function
    | "Periodic" -> Periodic
    | "Aperiodic" -> Aperiodic
    | "Sporadic" -> Sporadic
    | "Backgroud" -> Backgroud
    | str -> 
	failwith ("*** " ^ str ^ " is not a valid dispatch protocol.")

(** [get_dispatch_protocol c] returns dispatch protocol of the comp [c]. 
    Raises [Not_found] if no period is associated to [c]. *)
let (get_dispatch_protocol:  Aadl_ast.component -> dispatch_protocol) =
  fun component ->
    let period_id = Ident "Dispatch_Protocol" in
    let _, _, v, _ = 
      List.find (fun (id, _, _, _) -> id = period_id) component.comp_prop
    in
      begin match v with
	| [ Prop_expr_enumeration id ] -> 
	    string_to_dispatch_protocol (Ident.to_string id)
	| [] -> assert false
	| _ -> assert false
      end
    

type access_mode = Read | Write | ReadWrite | ByMethod

let (string_to_access_mode : string -> access_mode) =
  function
      "read" -> Read
    | "write" -> Write
    | "read_write" -> ReadWrite
    | "by_method" -> ByMethod
    | _ -> assert false

let (get_required_access_mode : Aadl_ast.property list -> access_mode) =
  fun pl -> 
    let id, prop_arrow, v, _opt = 
      List.find 
	(fun (id, _,_ , _) -> id = Ident "Required_Access") 
	pl
    in
      begin match v with
	| [ Prop_expr_enumeration mode ] -> 
	    string_to_access_mode (Ident.to_string mode)    
	| [] -> assert false
	| _ -> assert false
      end      

let (get_provided_access_mode : Aadl_ast.property list -> access_mode) =
  fun pl -> 
    let id, prop_arrow, v, _opt = 
      List.find 
	(fun (id, prop_arrow, mode, opt) -> id = Ident "Provided_Access") 
	pl
    in
      begin match v with
	| [ Prop_expr_enumeration mode ] -> 
	    string_to_access_mode (Ident.to_string mode)    
	| [] -> assert false
	| _ -> assert false
      end

